@@ -1,12 +1,10 @@
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0 OR ISC
 
-#if defined(BORINGSSL_FIPS)
-
 #include <gtest/gtest.h>
 
 #include "../../test/test_util.h"
-#include "../../../third_party/jitterentropy/jitterentropy.h"
+#include "../../../third_party/jitterentropy/jitterentropy-library/jitterentropy.h"
 
 // Struct for Jitter entropy collector instance with constructor/desctructor.
 struct JitterEC {
@@ -41,10 +39,8 @@ TEST(CPUJitterEntropyTest, Basic) {
   // Draw some entropy to check if it works.
   EXPECT_EQ(jent_read_entropy(jitter_ec.instance,
                               (char*) data0, data_len), data_len);
-
-  // Draw some entropy with the "safe" API to check if it works.
-  EXPECT_EQ(jent_read_entropy_safe(&jitter_ec.instance,
-                                   (char*) data1, data_len), data_len);
+  EXPECT_EQ(jent_read_entropy(jitter_ec.instance,
+                              (char*) data1, data_len), data_len);
 
   // Basic check that the random data is not equal.
   EXPECT_NE(Bytes(data0), Bytes(data1));
@@ -60,12 +56,10 @@ TEST(CPUJitterEntropyTest, Basic) {
   // Test drawing entropy from the Jitter object that was reset.
   EXPECT_EQ(jent_read_entropy(jitter_ec.instance,
                               (char*) data0, data_len), data_len);
-  EXPECT_EQ(jent_read_entropy_safe(&jitter_ec.instance,
-                                   (char*) data1, data_len), data_len);
+  EXPECT_EQ(jent_read_entropy(jitter_ec.instance,
+                              (char*) data1, data_len), data_len);
 
-  // Verify that the Jitter library version is v3.4.0.
-  unsigned int jitter_version = 3040000;
+  // Verify that the Jitter Entropy library version is as expected.
+  unsigned int jitter_version = 3060300;
   EXPECT_EQ(jitter_version, jent_version());
 }
-
-#endif
