@@ -28,6 +28,10 @@ int FIPS_mode(void) {
 #endif
 }
 
+int FIPS_is_entropy_cpu_jitter(void) {
+  return 1;
+}
+
 int FIPS_mode_set(int on) { return on == FIPS_mode(); }
 
 #if defined(BORINGSSL_FIPS_140_3)
@@ -62,6 +66,11 @@ int FIPS_query_algorithm_status(const char *algorithm) {
     "SHA2-384",
     "SHA2-512",
     "SHA2-512/256",
+    "SHA3-256",
+    "SHA3-384",
+    "SHA3-512",
+    "SHAKE128",
+    "SHAKE256",
   };
   for (size_t i = 0; i < OPENSSL_ARRAY_SIZE(kApprovedAlgorithms); i++) {
     if (strcmp(algorithm, kApprovedAlgorithms[i]) == 0) {
